; Dsassemblage de MIDIT32.BIN
; Interface Midipak de Logimus

; Dfinitions

PUTCH           EQU    $E803
KTSTH           EQU    $E809

US              EQU    $1F

MIDI.CTRL       EQU    $E7F2
MIDI.STAT       EQU    MIDI.CTRL
MIDI.RX         EQU    $E7F3
MIDI.TX         EQU    MIDI.RX

MIDI.RXFULL     EQU    $01
MIDI.TXEMPTY    EQU    $02
MIDI.RXIRQ      EQU    $80

MIDI.NOTEON     EQU    $90
MIDI.NOTEOFF    EQU    $80

CONN.OK         EQU    $01
CONN.Err        EQU    $02

NODISP          EQU    $01
BREAK           EQU    $01


                ORG    $B000

; Point d'entre inutilis
JumpToRM:
B000 7EB093     JMP    RecDispNotes             


; Routine pour attendre la rception d'un octet MIDI, peut tre interrompu
; par une touche au clavier
; Sortie : A = octet reu, BreakFlag = 1 si scan interrompu par le clavier
WaitByteOrKbd:
B003 B6E7F2     LDA    MIDI.STAT              
B006 8401       ANDA   #MIDI.RXFULL    ; Y'a du monde au portail ?    
B008 2704       BEQ    TestKbd              

B00A B6E7F3     LDA    MIDI.RX         ; Oui, rcuprons cet octet     
B00D 39         RTS                    ; Et on se barre dare-dare   

TestKbd:
B00E BDE809     JSR    KTSTH           ; Touche presse ?   
B011 24F0       BCC    WaitByteOrKbd   ; Non, on boucle   

B013 8601       LDA    #BREAK          ; Interrompu par le clavier   
B015 B7B144     STA    BreakFlag              
B018 39         RTS                       

GetNextByteD0:
B019 BDB003     JSR    WaitByteOrKbd              
B01C B7B142     STA    D0    
B01F 39         RTS                       

GetNextByteC:
B020 BDB003     JSR    WaitByteOrKbd              
B023 B7B143     STA    C              
B026 39         RTS                       

; Routine pour afficher une * en fonction de la note joue
; Entre : D0 = numro de note sur le clavier de l'instrument
DispStar:
B027 BDB039     JSR    CursPosFromNote ; Positionne le curseur             
B02A B6B149     LDA    NoDispFlag              
B02D 2605       BNE    ClearFlagReturn              

B02F C62A       LDB    #'*               
B031 BDB200     JSR    Display
  
ClearFlagReturn:            
B034 7FB149     CLR    NoDispFlag              
B037 39         RTS                       

; Routine qui ne fait rien.
DoNothing:
B038 39         RTS                       

; Route de positionnement du curseur  l'cran en fonctione de la note joue
; Entre : D0 contient le numro de note sur le clavier de l'instrument
CurPosFromNote:
B039 5F         CLRB                      
B03A B6B142     LDA    D0              ; Numro de note

DivBy12:              
B03D 810C       CMPA   #12               
B03F 2506       BCS    OctaveRange     ; Octave 0   

B041 800C       SUBA   #12               
B043 5C         INCB                   ; Incrmente quotient (= octave)   
B044 7EB03D     JMP    DivBy12         ; La note cherche sera le reste     

OctaveRange:
B047 C101       CMPB   #1              ; Octave doit tre entre 1 et 9   
B049 10250019   LBCS   NoteError              

B04D C109       CMPB   #9               
B04F 10220013   LBHI   NoteError              

B053 58         ASLB                   ; Calcule les coordonnes  l'cran   
B054 F7B06C     STB    CursY               
B057 B7B06E     STA    Tmp0            ; Note*3+1   
B05A 48         ASLA                      
B05B BBB06E     ADDA   Tmp0              
B05E 4C         INCA                      
B05F B7B06D     STA    CursX              
B062 BDB06F     JSR    Locate          ; Et positionne le curseur    
B065 39         RTS                       

NoteError:
B066 8601       LDA    #NODISP               
B068 B7B149     STA    NoDispFlag              
B06B 39         RTS                       

; Des variables qui trainent

CursY:
B06C            RMB    1               ; Ligne pour positionnement curseur

CursX:
B06D            RMB    1               ; Colonne pour positionnement curseur

Tmp0:
B06E            RMB    1               ; Variable temporaire de calcul              

; Routine LOCATE X,Y
; Entre : CursX = colonne, CursY = ligne
Locate:
B06F C61F       LDB    #US               
B071 BDB200     JSR    Display              
B074 F6B06C     LDB    CursY              
B077 CB40       ADDB   #$40               
B079 BDB200     JSR    Display              
B07C F6B06D     LDB    CursX              
B07F CB40       ADDB   #$40               
B081 BDB200     JSR    Display              
B084 39         RTS                                            

; Routine de dcomposition de l'octet d'entte de message MIDI
; Entre : A = Entte de message MIDI
; Sortie : A = Evnement, C0 = Entte de message complet, N = voie
Split:
B085 B7B140     STA    C0              ; Message MIDI
B088 840F       ANDA   #$0F               
B08A B7B141     STA    N               ; Sauve la voie
B08D B6B140     LDA    C0              
B090 84F0       ANDA   #$F0            ; Ne garde que l'vnement    
B092 39         RTS                       

; Routine pour visualiser des notes joues sur l'instrument
; Pas d'entre, pas de sortie; les variables de liaison au BASIC sont 
; utilises  des fins internes.
RecDispNotes:
B093 BDB204     JSR    ResetCtrl       ; Initialise le contrleur MIDI       
B096 7FB144     CLR    BreakFlag       ; Et les variables   
B099 7FB149     CLR    NoDispFlag

RecPollLoop:              
B09C BDB003     JSR    WaitByteOrKbd   ; Attend un octet           
B09F 7DB144     TST    BreakFlag              
B0A2 2654       BNE    RecDispEnd              

RecWhatIsIt:
B0A4 BDB085     JSR    Split           ; Dcompose l'octet d'entte   
B0A7 8190       CMPA   #MIDI.NOTEON    ; C'est un Note On ?           
B0A9 2707       BEQ    RecNoteOn              

B0AB 8180       CMPA   #MIDI.NOTEOFF   ; C'est un Note Off ?            
B0AD 272D       BEQ    RecNoteOff              

B0AF 7EB09C     JMP    RecPollLoop     ; Tout le reste, on jette         

RecNoteOn:
B0B2 BDB019     JSR    GetNextByteD0              

GetParams:
B0B5 BDB020     JSR    GetNextByteC              
B0B8 4D         TSTA                      
B0B9 271B       BEQ    RemoveNote      ; Vlocit 0 ? (eqv. Note Off)        

B0BB BDB027     JSR    DispStar        ; Affiche la note  l'cran      

NoteOnLoop:
B0BE BDB003     JSR    WaitByteOrKbd              
B0C1 7DB144     TST    BreakFlag              
B0C4 2632       BNE    RecDispEnd              

B0C6 B7B140     STA    C0              ; Sauve l'octet
B0C9 8580       BITA   #$80            ; Dbut de message MIDI ?   
B0CB 26D7       BNE    RecWhatIsIt     ; Oui -> on l'interprte         

B0CD B6B140     LDA    C0              ; Rcupre l'octet
B0D0 B7B142     STA    D0              
B0D3 7EB0B5     JMP    GetParams       ; Ignore le reste du message   

RemoveNote:
B0D6 BDB038     JSR    DoNothing              
B0D9 7EB0BE     JMP    NoteOnLoop              

RecNoteOff:
B0DC BDB019     JSR    GetNextByteD0         
     
NoteOffLoop:     
B0DF BDB020     JSR    GetNextByteC              
B0E2 BDB038     JSR    DoNothing              
B0E5 BDB003     JSR    WaitByteOrKbd              
B0E8 B7B140     STA    C0              ; Sauve l'octet
B0EB 8580       BITA   #$80            ; Dbut de message MIDI ?   
B0ED 26B5       BNE    RecWhatIsIt     ; Oui -> on l'interprte   

B0EF B6B140     LDA    C0              ; Rcupre l'octet
B0F2 B7B142     STA    D0              
B0F5 7EB0DF     JMP    NoteOffLoop              

RecDispEnd:
B0F8 39         RTS                    ; Retour au BASIC   

; ------------------------------------------------------------------------
Unused0:
B0F9            RMB    $B100-*          ; 7 octets inutiliss
; ------------------------------------------------------------------------

                ORG    $B100

; Points d'entre appelables depuis le BASIC avec EXEC

I0:
B100 7EB204     JMP    ResetCtrl       ; Rinitialisation simple  

O0:
B103 7EB20F     JMP    SendByte        ; Envoi d'un octet (MIDI Out)   

M0:
B106 7EB21E     JMP    PlayNote        ; Joue une note sur l'instrument      

M1:
B109 7EB22A     JMP    PlayNoteVel0    ; Stoppe une note sur l'instrument          

T0:
B10C 7EB246     JMP    LinkTest        ; Teste la liaison MIDI      

V0:
B10F 7EB093     JMP    RecDispNotes    ; Rception/visu de notes joues         

; ------------------------------------------------------------------------
Unused1:
B110            RMB    $B140-*         ; Plein de RAM inutilise
; ------------------------------------------------------------------------

                ORG    $B140

; Variables d'interface avec le BASIC (C0, N, D0, C)
C0:
Channel:           
B140            RMB    1               ; Numro de canal 

N:
KeyNb:
B141            RMB    1               ; Touche  presser 

D0:
Velocity:
B142            RMB    1               ; Vlocit 

C:
RetCode:
B143            RMB    1               ; Code de retour (CONN.xx) 

; Variables internes
BreakFlag:
B144            RMB    1               ; Mis  1 quand polling interrompu

Unused2:
B145            RMB    4               ; Encore des octets de gchs

NoDispFlag:
B149            RMB    1               ; Mis  1 pour viter affichage

; ------------------------------------------------------------------------
Unused3
B14A            RMB    $B200-*         ; Plein de RAM inutilise        
; ------------------------------------------------------------------------

                ORG    $B200

; Routine d'affichage d'un caractre
; Entre : B = caractre  afficher
Display:
B200 BDE803     JSR    PUTCH            
B203 39         RTS                       

; Routine de rinitialisation simple du contrleur MIDI
ResetCtrl:
B204 8603       LDA    #$03            ; Reset contrleur MIDI   
B206 B7E7F2     STA    MIDI.CTRL              
B209 8615       LDA    #$15               
B20B B7E7F2     STA    MIDI.CTRL       ; Lecture/criture sans interruptions       
B20E 39         RTS                       

; Routine d'envoi d'un octet sur l'interface MIDI
; Entre : A = octet  envoyer 
SendByte:
B20F 3402       PSHS   A     
TXRdyLoop:             
B211 B6E7F2     LDA    MIDI.STAT       ; Attend que l'interface est prte       
B214 8402       ANDA   #MIDI.TXEMPTY   ;  envoyer            
B216 27F9       BEQ    TXRdyLoop              

B218 3502       PULS   A                  
B21A B7E7F3     STA    MIDI.TX         ; Envoi de l'octet  
B21D 39         RTS                       

; Routine pour jouer une note sur l'instrument
; Entre: Channel = voie, KeyNb = numro de touche, Velocity = vlocit
PlayNote:
B21E BDB232     JSR    SendNoteOn      ; Envoie Note On        
B221 B6B142     LDA    Velocity              
B224 847F       ANDA   #$7F               
B226 BDB20F     JSR    SendByte        ; Envoie vlocit      
B229 39         RTS                       

; Routine pour jouer une note de vlocit 0 (eqv. Note Off)
; Entre: Channel = voie, KeyNb = numro de touche
PlayNoteVel0:
B22A BDB232     JSR    SendNoteOn      ; Note On        
B22D 4F         CLRA                   ; avec velocit 0 (eqv. Note Off)   
B22E BDB20F     JSR    SendByte              
B231 39         RTS                       

; Route d'envoi de message Note On, la vlocit est  envoyer en plus
; Entre : Channel = voie, KeyNb = numro de touche
SendNoteOn:
B232 B6B140     LDA    Channel              
B235 4A         DECA                      
B236 840F       ANDA   #$0F               
B238 8B90       ADDA   #MIDI.NOTEON               
B23A BDB20F     JSR    SendByte        ; Envoie message Note On      
B23D B6B141     LDA    KeyNb              
B240 847F       ANDA   #$7F               
B242 BDB20F     JSR    SendByte        ; Envoie numro de touche      
B245 39         RTS                       

; Routine pour tester la liaison MIDI. Envoie un code $55 et en attend un cho
; Retour : RetCode = 1 si liaison OK, 2 si liaison dfectueuse
LinkTest:
B246 BDB204     JSR    ResetCtrl              
B249 B6E7F3     LDA    MIDI.RX              
B24C 8655       LDA    #$55            ; Envoie octet d'cho   
B24E BDB20F     JSR    SendByte              
B251 C6FF       LDB    #$FF   

LinkWaitLoop:            
B253 5A         DECB                   ; Attend un peu   
B254 26FD       BNE    LinkWaitLoop              

B256 B6E7F2     LDA    MIDI.STAT              
B259 8401       ANDA   #RXFULL               
B25B 270F       BEQ    LinkError       ; Rien reu aprs tout ce temps ?       

B25D B6E7F3     LDA    MIDI.RX              
B260 8155       CMPA   #$55               
B262 2608       BNE    LinkError       ; Octet reu pas un cho ?       

B264 8601       LDA    #CONN.OK        ; Si, c'est bon       
B266 B7B143     STA    RetCode              
B269 7EB271     JMP    LinkReturn              

LinkError:
B26C 8602       LDA    #$CONN.Err      ; Non, c'est fichu         
B26E B7B143     STA    RetCode 
  
LinkReturn:           
B271 39         RTS                       

